"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setCsrfCookies = setCsrfCookies;
exports.getCsrfPlugin = getCsrfPlugin;
exports.getAppRoutePlugin = getAppRoutePlugin;
exports.getLoggerPlugin = getLoggerPlugin;
exports.getActivityTrackingPlugin = getActivityTrackingPlugin;
exports.default = getPlugins;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _inert = _interopRequireDefault(require("inert"));

var _csrfHapi = _interopRequireDefault(require("electrode-csrf-jwt/lib/csrf-hapi17"));

var _mwpConfig = _interopRequireDefault(require("mwp-config"));

var _mwpLoggerPlugin = require("mwp-logger-plugin");

var _mwpAppRoutePlugin = require("mwp-app-route-plugin");

var _activity = require("mwp-tracking-plugin/lib/activity");

var _click = require("mwp-tracking-plugin/lib/click");

var _mwpLanguagePlugin = require("mwp-language-plugin");

var _mwpSwPlugin = require("mwp-sw-plugin");

var _mwpApiProxyPlugin = require("mwp-api-proxy-plugin");

var _mwpAuthPlugin = require("mwp-auth-plugin");

/**
 * Hapi plugins for the dev server
 *
 * @module ServerPlugins
 */
var CSRF_COOKIE_NAME = process.env.NODE_ENV === 'production' ? 'x-mwp-csrf' : 'x-mwp-csrf_dev';
var CSRF_HEADER_COOKIE_NAME = "".concat(CSRF_COOKIE_NAME, "-header");
var CSRF_HEADER_NAME = CSRF_COOKIE_NAME; // Sets the csrf header token from plugin into a cookie

function setCsrfCookies(request, h) {
  var csrfHeader = (request.response.headers || {})[CSRF_COOKIE_NAME];

  if (csrfHeader) {
    h.state(CSRF_HEADER_COOKIE_NAME, csrfHeader, {
      isSecure: true
    });
  }

  return h.continue;
}
/**
 * The CSRF plugin we use, 'electrode-csrf-jwt', generates a token for each request
 * that we make and sets the token in an HTTP-only cookie (CSRF_COOKIE_NAME) and in
 * the HTTP response header (also CSRF_COOKIE_NAME). In non-GET requests we must supply
 * the latest generated token from the response header as an HTTP request header - the
 * plugin will compare the cookie token and the request header token and return a
 * BAD_TOKEN error if they do not match.
 *
 * We updated this flow to set the token from the response header as a cookie
 * (CSRF_COOKIE_NAME-header) and use the cookie value to set the request header
 * so that it syncs across browser tabs.
 *
 * We set similar but different cookie names for dev and prod environments so the prod
 * cookies are not read in the dev environment.
 *
 * In order to ensure that both cookie values have parallel settings, this
 * function calls `server.state` for both cookie names before registering the
 * plugin.
 *
 * @return {Object} the { register } object for a `server.register` call.
 */


function getCsrfPlugin(electrodeOptions) {
  var register = function register(server, options) {
    var isProd = server.settings.app.isProd;
    var cookieOptions = {
      path: '/',
      isSecure: isProd,
      // No need to worry about https in dev
      isSameSite: false,
      // Firefox will not read SameSite cookies set on redirect (e.g. from email link), so we disable that setting
      domain: isProd ? '.meetup.com' : '.dev.meetup.com' // target the current app server domain

    };
    Object.assign(options, {
      secret: server.settings.app.csrf_secret
    }, electrodeOptions);
    server.state(CSRF_COOKIE_NAME, // set by plugin
    (0, _objectSpread2.default)({}, cookieOptions, {
      isHttpOnly: true // no client-side interaction needed

    }));
    server.state(CSRF_HEADER_COOKIE_NAME, // set by onPreResponse
    (0, _objectSpread2.default)({}, cookieOptions, {
      isHttpOnly: false // the client must read this cookie and return as a custom header

    }));

    var registration = _csrfHapi.default.register(server, options);

    server.ext('onPreResponse', setCsrfCookies); // this extension must be registered _after_ plugin is registered

    return registration;
  };

  return {
    plugin: {
      register: register,
      pkg: _csrfHapi.default.pkg
    }
  };
}

function getAppRoutePlugin(options) {
  return {
    plugin: _mwpAppRoutePlugin.plugin,
    options: options
  };
}

function getLoggerPlugin() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    logEvents: ['onPostStart', 'onPostStop', 'response']
  };
  return {
    plugin: _mwpLoggerPlugin.plugin,
    options: options
  };
}

function getActivityTrackingPlugin(_ref) {
  var agent = _ref.agent,
      isProdApi = _ref.isProdApi;
  return {
    plugin: _activity.plugin,
    options: {
      agent: agent,
      isProdApi: isProdApi
    }
  };
}

function getPlugins(_ref2) {
  var languageRenderers = _ref2.languageRenderers;
  var agent = _mwpConfig.default.package.agent,
      getServer = _mwpConfig.default.getServer;
  var server = getServer();
  var isProdApi = server.properties.api.isProd;
  return [getAppRoutePlugin({
    languageRenderers: languageRenderers
  }), _mwpApiProxyPlugin.plugin, _mwpLanguagePlugin.plugin, getLoggerPlugin(), getCsrfPlugin({
    headerName: CSRF_HEADER_NAME,
    cookieName: CSRF_COOKIE_NAME
  }), _mwpAuthPlugin.plugin, getActivityTrackingPlugin({
    agent: agent,
    isProdApi: isProdApi
  }), _click.plugin, _mwpSwPlugin.plugin, _inert.default];
}
//# sourceMappingURL=getPlugins.js.map